package com.dsic.gj_erp.service.xiaochi;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.cron.CronUtil;
import cn.hutool.cron.task.Task;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.xiaochi.CutPlan;
import com.dsic.gj_erp.bean.xiaochi.Device;
import com.dsic.gj_erp.bean.xiaochi.XiaochiUser;
import com.dsic.gj_erp.mapper.xiaochi.CutPlanMapper;
import com.dsic.gj_erp.service.zyjh.ZyjhService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@AllArgsConstructor
public class XiaoChiService{

    private final ZyjhService zyjhService;
    private final CutPlanMapper mapper;


    /**
     * 小池同步,先使用手动拉取方式,让钢加人员进行检查
     * 后续改为自动拉取
     */
    @PostConstruct
    public void handler(){
        List<XiaochiUser> users = new XiaochiUser().selectAll();
        List<Device> devices=new Device().selectAll();
        Map<String, XiaochiUser> userMap=new HashMap<>();
        users.forEach(item->{
            userMap.put(item.getEmployeeNumber(),item);
        });
        Map<Integer,Device> deviceMap=new HashMap<>();
        devices.forEach(item-> deviceMap.put(item.getId(),item));
        log.info("========开始执行小池同步定时功能=========");
        this.exeAuto(userMap,deviceMap);
        //this.exe(userMap,deviceMap);
    }

    private void exe(Map<String, XiaochiUser> userMap,Map<Integer,Device> deviceMap){
        log.info("========小池同步开始===========");
        List<CutPlan> cutPlans = mapper.getCutPlanList(DateUtil.lastMonth().toString(),DateUtil.today())
                .stream().filter(item->StrUtil.isNotEmpty(item.getNestKakuyakuDate())).collect(Collectors.toList());
        List<DmYdjh> list=new ArrayList<>();
        cutPlans.forEach(item->{
            DmYdjh one = zyjhService.getOne(Wrappers.<DmYdjh>lambdaQuery()
                    .eq(DmYdjh::getDcCh, item.getNestExtC02())
                    .eq(DmYdjh::getTzbh, item.getNestNameSub1())
                    .eq(DmYdjh::getDcPl, item.getNestExtC04())
            );
            if (ObjectUtil.isNotEmpty(one)){
                DmYdjh _item=new DmYdjh();
                _item.setId(one.getId());
                if (item.getNestCutLengs()>0){
                    Optional.of(userMap.get(item.getNestExtC05()))
                            .ifPresent(user-> {
                                _item.setQgfkry(userMap.get(item.getNestExtC05()).getEmployeeNumber());
                            });
                    try {
                        DateTime qgkssj = DateUtil.parse(item.getQgkssj());
                        _item.setQgpgrq(qgkssj.toString("yyyy/MM/dd HH:mm:ss"));
                    }catch (Exception ignored){}

                    try {
                        DateTime qgjssj = DateUtil.parse(item.getQgjssj());
                        _item.setQgfkrq(qgjssj.toString("yyyy/MM/dd HH:mm:ss"));
                    }catch (Exception ignored){}
                    _item.setZtForQGFK();
                }

                if (item.getNestMarkLengs()>0){
                    Optional.of(userMap.get(item.getNestExtC05()))
                            .ifPresent(user-> {
                                _item.setHxfkry(userMap.get(item.getNestExtC05()).getEmployeeNumber());

                            });
                    try {
                        DateTime qgkssj = DateUtil.parse(item.getQgkssj());
                        _item.setHxpgrq(qgkssj.toString("yyyy/MM/dd HH:mm:ss"));
                    }catch (Exception ignored){}

                    try {
                        DateTime qgjssj = DateUtil.parse(item.getQgjssj());
                        _item.setHxfkrq(qgjssj.toString("yyyy/MM/dd HH:mm:ss"));
                    }catch (Exception ignored){}
                    _item.setZtForHXFK();
                }
                list.add(_item);
            }
        });
        zyjhService.updateBatchById(list,500);
        log.info("========小池同步完成===========");
    }

    private void exeAuto(Map<String, XiaochiUser> userMap,Map<Integer,Device> deviceMap){
        //CronUtil默认为分钟,目前未使用CronUtil.setMatchSecond(true)启动秒级任务,以下表示每30分钟执行一次
        //每30分钟执行一次,获取小池今日加工并且未更新切割计划派工单的数据去更新派工单
        CronUtil.schedule("0 */10 * * * ?", (Task) () -> {
            this.exe(userMap,deviceMap);
        });
        CronUtil.setMatchSecond(true);
        if (!CronUtil.getScheduler().isStarted()){
            CronUtil.start();
        }
    }

//    @PostConstruct
    public  void synBom_qgPgd(){
        //每天凌晨一点，同步前一天数据
//        CronUtil.schedule("0 1 * * *", (Task) () -> {
        CronUtil.schedule("*/1 * * * * *", (Task) () -> {
            mapper.synBom_qgPgd();
        });
        CronUtil.start();
    }

}
